==> ugene_1.3.3+repack.orig.tar.gz: ugene-1.3.3/src/core/src/util_ov_annotated_dna/GSequenceLineView.h <==
/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_GSEQUENCE_LINE_VIEW_H_
#define _GB2_GSEQUENCE_LINE_VIEW_H_

#include <core_api/LRegion.h>

#include <QtCore/QFlag>
#include <QtGui/QWidget>
#include <QtGui/QMouseEvent>
#include <QtGui/QWheelEvent>
#include <QtGui/QFocusEvent>
#include <QtGui/QPainter>
#include <QtGui/QMenu>


namespace GB2 {

class GSequenceLineViewRenderArea;
class DNASequenceObject;
class DNASequenceSelection;
class LRegionsSelection;
class GScrollBar;
class ADVSequenceObjectContext;
class GObjectViewOpConstraints;
class GObject;

enum GSLV_UpdateFlag {
    GSLV_UF_NeedCompleteRedraw  = 1<<0,
    GSLV_UF_ViewResized         = 1<<1,
    GSLV_UF_VisibleRangeChanged = 1<<2,
    GSLV_UF_SelectionChanged    = 1<<3,
    GSLV_UF_FocusChanged        = 1<<4,
    GSLV_UF_FrameChanged        = 1<<5,
    GSLV_UF_AnnotationsChanged  = 1<<6
};

enum GSLV_FeatureFlag {
    GSLV_FF_SupportsCustomRange = 0x1
};

typedef QFlags<GSLV_UpdateFlag> GSLV_UpdateFlags;
typedef QFlags<GSLV_FeatureFlag> GSLV_FeatureFlags;

//single-line sequence view
class GB2_COREAPI_EXPORT GSequenceLineView : public QWidget {
	Q_OBJECT
public:
	GSequenceLineView(QWidget* p, ADVSequenceObjectContext* ctx);

	const LRegion& getVisibleRange() const {return visibleRange;}

	ADVSequenceObjectContext* getSequenceContext() const {return ctx;}


	virtual void setStartPos(int pos);
	
    virtual void setCenterPos(int pos);

	int getSequenceLen() const {return seqLen;}
    
    virtual void addUpdateFlags(GSLV_UpdateFlags newFlags) {lastUpdateFlags|=newFlags;}
       
    virtual void clearUpdateFlags() {lastUpdateFlags = 0;}

    GSLV_UpdateFlags getUpdateFlags() const {return lastUpdateFlags;}

    virtual void setFrameView(GSequenceLineView* frameView); 

    virtual GSequenceLineView* getFrameView() const {return frameView;}
    
    virtual void setConherentRangeView(GSequenceLineView* rangeView);

    virtual GSequenceLineView* getConherentRangeView() const {return coherentRangeView;}

    // [0..seqLen)
    virtual void setVisibleRange(const LRegion& reg, bool signal = true);

    virtual QAction* getZoomInAction() const {return coherentRangeView == NULL ? NULL : coherentRangeView->getZoomInAction();}

    virtual QAction* getZoomOutAction() const {return coherentRangeView == NULL ? NULL : coherentRangeView->getZoomOutAction();}

    virtual QAction* getZoomToSelectionAction() const {return coherentRangeView == NULL ? NULL : coherentRangeView->getZoomToSelectionAction();}

    virtual QAction* getZoomToSequenceAction() const {return coherentRangeView == NULL ? NULL : coherentRangeView->getZoomToSequenceAction();}
    
    virtual DNASequenceObject* getSequenceObject() const;

    virtual void buildPopupMenu(QMenu& m){ Q_UNUSED(m); }

    virtual bool isWidgetOnlyObject(GObject* o) const { Q_UNUSED(o); return false;}

signals:
	void si_visibleAreaChanged(GSequenceLineView* v);
    void si_centerPosition(int pos);

protected:
	void resizeEvent(QResizeEvent* e);
	void mouseDoubleClickEvent(QMouseEvent* me);
	void mousePressEvent(QMouseEvent* me);
	void mouseReleaseEvent(QMouseEvent* me);
	void mouseMoveEvent(QMouseEvent* me);
	void wheelEvent(QWheelEvent* we);
	void focusInEvent(QFocusEvent* fe);
	void focusOutEvent(QFocusEvent* fe);
    void keyPressEvent(QKeyEvent *e);
	
	virtual void onVisibleRangeChanged(bool signal = true);

public slots:
    void sl_centerPosition(int pos) {setCenterPos(pos);}

protected slots:
	virtual void sl_onScrollBarMoved(int pos);
	virtual void sl_onDNASelectionChanged(LRegionsSelection* thiz, const QList<LRegion>& added, const QList<LRegion>& removed);
    void sl_onFrameRangeChanged(GSequenceLineView* dv);
    void sl_onCoherentRangeViewRangeChanged(GSequenceLineView*);
    

protected:
	QPoint toRenderAreaPoint(const QPoint& p);
	void updateScrollBar();
	void setSelection(const LRegion& r);
	virtual void pack();
	virtual int getSingleStep() const; 
	virtual int getPageStep() const; 
	
	ADVSequenceObjectContext*	    ctx;
	GSequenceLineViewRenderArea*	renderArea;
	LRegion							visibleRange;
	GScrollBar*						scrollBar;
	int								lastPressPos;
	int								seqLen;
    GSLV_UpdateFlags                lastUpdateFlags;
    GSLV_FeatureFlags               featureFlags;
    
    GSequenceLineView*              frameView;
    GSequenceLineView*              coherentRangeView;
};

class GB2_COREAPI_EXPORT GSequenceLineViewRenderArea : public QWidget  {
public:
	GSequenceLineViewRenderArea(GSequenceLineView* p);
	~GSequenceLineViewRenderArea();

	virtual int coordToPos(int x) const;
    virtual int posToCoord(int p, bool useVirtualSpace = false) const;
    virtual float posToCoordF(int p, bool useVirtualSpace = false) const;
    //number of pixels per base
    virtual float getCurrentScale() const;


protected:
	void paintEvent(QPaintEvent *e);
    
	virtual void drawAll(QPaintDevice* pd) = 0;
    void drawFrame(QPainter& p);
	virtual void drawFocus(QPainter& p);

	GSequenceLineView* view;
	QPixmap* cachedView;
    QPixmap* tmpView;
};

} //namespace

#endif

==> ugene_1.5.2+repack.orig.tar.gz: ugene-1.5.2/src/core/src/util_ov_annotated_dna/GSequenceLineView.h <==
/*****************************************************************
* Unipro UGENE - Integrated Bioinformatics Suite
* Copyright (C) 2008,2009 Unipro, Russia (http://ugene.unipro.ru)
* All Rights Reserved
* 
*     This source code is distributed under the terms of the
*     GNU General Public License. See the files COPYING and LICENSE
*     for details.
*****************************************************************/

#ifndef _GB2_GSEQUENCE_LINE_VIEW_H_
#define _GB2_GSEQUENCE_LINE_VIEW_H_

#include <core_api/LRegion.h>

#include <QtCore/QFlag>
#include <QtGui/QWidget>
#include <QtGui/QMouseEvent>
#include <QtGui/QWheelEvent>
#include <QtGui/QFocusEvent>
#include <QtGui/QPainter>
#include <QtGui/QMenu>


namespace GB2 {

class GSequenceLineViewRenderArea;
class DNASequenceObject;
class DNASequenceSelection;
class LRegionsSelection;
class GScrollBar;
class ADVSequenceObjectContext;
class GObjectViewOpConstraints;
class GObject;

enum GSLV_UpdateFlag {
    GSLV_UF_NeedCompleteRedraw  = 1<<0,
    GSLV_UF_ViewResized         = 1<<1,
    GSLV_UF_VisibleRangeChanged = 1<<2,
    GSLV_UF_SelectionChanged    = 1<<3,
    GSLV_UF_FocusChanged        = 1<<4,
    GSLV_UF_FrameChanged        = 1<<5,
    GSLV_UF_AnnotationsChanged  = 1<<6
};

enum GSLV_FeatureFlag {
    GSLV_FF_SupportsCustomRange = 0x1
};

typedef QFlags<GSLV_UpdateFlag> GSLV_UpdateFlags;
typedef QFlags<GSLV_FeatureFlag> GSLV_FeatureFlags;

//single-line sequence view
class GB2_COREAPI_EXPORT GSequenceLineView : public QWidget {
	Q_OBJECT
public:
	GSequenceLineView(QWidget* p, ADVSequenceObjectContext* ctx);

	const LRegion& getVisibleRange() const {return visibleRange;}

	ADVSequenceObjectContext* getSequenceContext() const {return ctx;}


	virtual void setStartPos(int pos);
	
    virtual void setCenterPos(int pos);

	int getSequenceLen() const {return seqLen;}
    
    virtual void addUpdateFlags(GSLV_UpdateFlags newFlags) {lastUpdateFlags|=newFlags;}
       
    virtual void clearUpdateFlags() {lastUpdateFlags = 0;}

    GSLV_UpdateFlags getUpdateFlags() const {return lastUpdateFlags;}

    virtual void setFrameView(GSequenceLineView* frameView); 

    virtual GSequenceLineView* getFrameView() const {return frameView;}
    
    virtual void setConherentRangeView(GSequenceLineView* rangeView);

    virtual GSequenceLineView* getConherentRangeView() const {return coherentRangeView;}

    // [0..seqLen)
    virtual void setVisibleRange(const LRegion& reg, bool signal = true);

    virtual QAction* getZoomInAction() const {return coherentRangeView == NULL ? NULL : coherentRangeView->getZoomInAction();}

    virtual QAction* getZoomOutAction() const {return coherentRangeView == NULL ? NULL : coherentRangeView->getZoomOutAction();}

    virtual QAction* getZoomToSelectionAction() const {return coherentRangeView == NULL ? NULL : coherentRangeView->getZoomToSelectionAction();}

    virtual QAction* getZoomToSequenceAction() const {return coherentRangeView == NULL ? NULL : coherentRangeView->getZoomToSequenceAction();}
    
    virtual DNASequenceObject* getSequenceObject() const;

    virtual void buildPopupMenu(QMenu& m){ Q_UNUSED(m); }

    virtual bool isWidgetOnlyObject(GObject* o) const { Q_UNUSED(o); return false;}

signals:
	void si_visibleRangeChanged();
    void si_centerPosition(int pos);

protected:
	void resizeEvent(QResizeEvent* e);
	void mouseDoubleClickEvent(QMouseEvent* me);
	void mousePressEvent(QMouseEvent* me);
	void mouseReleaseEvent(QMouseEvent* me);
	void mouseMoveEvent(QMouseEvent* me);
	void wheelEvent(QWheelEvent* we);
	void focusInEvent(QFocusEvent* fe);
	void focusOutEvent(QFocusEvent* fe);
    void keyPressEvent(QKeyEvent *e);
    
	virtual void onVisibleRangeChanged(bool signal = true);

public slots:
    void sl_centerPosition(int pos) {setCenterPos(pos);}

protected slots:
	virtual void sl_onScrollBarMoved(int pos);
	virtual void sl_onDNASelectionChanged(LRegionsSelection* thiz, const QList<LRegion>& added, const QList<LRegion>& removed);
    virtual void sl_sequenceChanged();
    void sl_onFrameRangeChanged();
    void sl_onCoherentRangeViewRangeChanged();
    void completeUpdate(); 

protected:
	QPoint toRenderAreaPoint(const QPoint& p);
	void updateScrollBar();
	void setSelection(const LRegion& r);
	virtual void pack();
	virtual int getSingleStep() const; 
	virtual int getPageStep() const; 
	
	ADVSequenceObjectContext*	    ctx;
	GSequenceLineViewRenderArea*	renderArea;
	LRegion							visibleRange;
	GScrollBar*						scrollBar;
	int								lastPressPos;
	int								seqLen;
    GSLV_UpdateFlags                lastUpdateFlags;
    GSLV_FeatureFlags               featureFlags;
    GSequenceLineView*              frameView;
    GSequenceLineView*              coherentRangeView;
};

class GB2_COREAPI_EXPORT GSequenceLineViewRenderArea : public QWidget  {
public:
	GSequenceLineViewRenderArea(GSequenceLineView* p);
	~GSequenceLineViewRenderArea();

	virtual int coordToPos(int x) const;
    virtual int posToCoord(int p, bool useVirtualSpace = false) const;
    virtual float posToCoordF(int p, bool useVirtualSpace = false) const;
    //number of pixels per base
    virtual float getCurrentScale() const;
    //char width, derived from current 'font'
    int getCharWidth() const {return charWidth;}

protected:
	void paintEvent(QPaintEvent *e);
    
	virtual void drawAll(QPaintDevice* pd) = 0;
    void drawFrame(QPainter& p);
	virtual void drawFocus(QPainter& p);

    void updateFontMetrics();

	GSequenceLineView* view;
	QPixmap* cachedView;
    QPixmap* tmpView;

    //per char and per line metrics
    QFont			sequenceFont;
    QFont			smallSequenceFont;
    QFont			rulerFont;
    
    int				charWidth;
    int				smallCharWidth;

    int				lineHeight;
    int				yCharOffset;
    int				xCharOffset;


};

} //namespace

#endif
